import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { CancellationTokenSource } from 'vscode/vscode/vs/base/common/cancellation';
import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { isCancellationError } from 'vscode/vscode/vs/base/common/errors';
import { Event } from 'vscode/vscode/vs/base/common/event';
import { DisposableStore } from 'vscode/vscode/vs/base/common/lifecycle';
import { registerEditorContribution, EditorAction2 } from 'vscode/vscode/vs/editor/browser/editorExtensions';
import { ICodeEditorService } from 'vscode/vscode/vs/editor/browser/services/codeEditorService';
import { Range } from 'vscode/vscode/vs/editor/common/core/range';
import { PeekContext } from 'vscode/vscode/vs/editor/contrib/peekView/browser/peekView';
import { localize, localize2 } from 'vscode/vscode/vs/nls';
import { registerAction2, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { RawContextKey, ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IContextKeyService } from 'vscode/vscode/vs/platform/contextkey/common/contextkey.service';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { IStorageService } from 'vscode/vscode/vs/platform/storage/common/storage.service';
import { TypeHierarchyTreePeekWidget } from './typeHierarchyPeek.js';
import { TypeHierarchyProviderRegistry, TypeHierarchyModel } from 'vscode/vscode/vs/workbench/contrib/typeHierarchy/common/typeHierarchy';

var TypeHierarchyController_1;
const _ctxHasTypeHierarchyProvider = ( (new RawContextKey('editorHasTypeHierarchyProvider', false, ( localize(4251, 'Whether a type hierarchy provider is available')))));
const _ctxTypeHierarchyVisible = ( (new RawContextKey('typeHierarchyVisible', false, ( localize(4252, 'Whether type hierarchy peek is currently showing')))));
const _ctxTypeHierarchyDirection = ( (new RawContextKey(
    'typeHierarchyDirection',
    undefined,
    { type: 'string', description: ( localize(4253, 'whether type hierarchy shows super types or subtypes')) }
)));
function sanitizedDirection(candidate) {
    return candidate === "subtypes"  || candidate === "supertypes"
        ? candidate
        : "subtypes" ;
}
let TypeHierarchyController = class TypeHierarchyController {
    static { TypeHierarchyController_1 = this; }
    static { this.Id = 'typeHierarchy'; }
    static get(editor) {
        return editor.getContribution(TypeHierarchyController_1.Id);
    }
    static { this._storageDirectionKey = 'typeHierarchy/defaultDirection'; }
    constructor(_editor, _contextKeyService, _storageService, _editorService, _instantiationService) {
        this._editor = _editor;
        this._contextKeyService = _contextKeyService;
        this._storageService = _storageService;
        this._editorService = _editorService;
        this._instantiationService = _instantiationService;
        this._disposables = ( (new DisposableStore()));
        this._sessionDisposables = ( (new DisposableStore()));
        this._ctxHasProvider = _ctxHasTypeHierarchyProvider.bindTo(this._contextKeyService);
        this._ctxIsVisible = _ctxTypeHierarchyVisible.bindTo(this._contextKeyService);
        this._ctxDirection = _ctxTypeHierarchyDirection.bindTo(this._contextKeyService);
        this._disposables.add(Event.any(_editor.onDidChangeModel, _editor.onDidChangeModelLanguage, TypeHierarchyProviderRegistry.onDidChange)(() => {
            this._ctxHasProvider.set(_editor.hasModel() && ( (TypeHierarchyProviderRegistry.has(_editor.getModel()))));
        }));
        this._disposables.add(this._sessionDisposables);
    }
    dispose() {
        this._disposables.dispose();
    }
    async startTypeHierarchyFromEditor() {
        this._sessionDisposables.clear();
        if (!this._editor.hasModel()) {
            return;
        }
        const document = this._editor.getModel();
        const position = this._editor.getPosition();
        if (!( (TypeHierarchyProviderRegistry.has(document)))) {
            return;
        }
        const cts = ( (new CancellationTokenSource()));
        const model = TypeHierarchyModel.create(document, position, cts.token);
        const direction = sanitizedDirection(this._storageService.get(TypeHierarchyController_1._storageDirectionKey, 0 , "subtypes" ));
        this._showTypeHierarchyWidget(position, direction, model, cts);
    }
    _showTypeHierarchyWidget(position, direction, model, cts) {
        this._ctxIsVisible.set(true);
        this._ctxDirection.set(direction);
        Event.any(this._editor.onDidChangeModel, this._editor.onDidChangeModelLanguage)(this.endTypeHierarchy, this, this._sessionDisposables);
        this._widget = this._instantiationService.createInstance(TypeHierarchyTreePeekWidget, this._editor, position, direction);
        this._widget.showLoading();
        this._sessionDisposables.add(this._widget.onDidClose(() => {
            this.endTypeHierarchy();
            this._storageService.store(TypeHierarchyController_1._storageDirectionKey, this._widget.direction, 0 , 0 );
        }));
        this._sessionDisposables.add({ dispose() { cts.dispose(true); } });
        this._sessionDisposables.add(this._widget);
        model.then(model => {
            if (cts.token.isCancellationRequested) {
                return;
            }
            if (model) {
                this._sessionDisposables.add(model);
                this._widget.showModel(model);
            }
            else {
                this._widget.showMessage(( localize(4254, "No results")));
            }
        }).catch(err => {
            if (isCancellationError(err)) {
                this.endTypeHierarchy();
                return;
            }
            this._widget.showMessage(( localize(4255, "Failed to show type hierarchy")));
        });
    }
    async startTypeHierarchyFromTypeHierarchy() {
        if (!this._widget) {
            return;
        }
        const model = this._widget.getModel();
        const typeItem = this._widget.getFocused();
        if (!typeItem || !model) {
            return;
        }
        const newEditor = await this._editorService.openCodeEditor({ resource: typeItem.item.uri }, this._editor);
        if (!newEditor) {
            return;
        }
        const newModel = model.fork(typeItem.item);
        this._sessionDisposables.clear();
        TypeHierarchyController_1.get(newEditor)?._showTypeHierarchyWidget(Range.lift(newModel.root.selectionRange).getStartPosition(), this._widget.direction, Promise.resolve(newModel), ( (new CancellationTokenSource())));
    }
    showSupertypes() {
        this._widget?.updateDirection("supertypes" );
        this._ctxDirection.set("supertypes" );
    }
    showSubtypes() {
        this._widget?.updateDirection("subtypes" );
        this._ctxDirection.set("subtypes" );
    }
    endTypeHierarchy() {
        this._sessionDisposables.clear();
        this._ctxIsVisible.set(false);
        this._editor.focus();
    }
};
TypeHierarchyController = TypeHierarchyController_1 = ( (__decorate([
    ( (__param(1, IContextKeyService))),
    ( (__param(2, IStorageService))),
    ( (__param(3, ICodeEditorService))),
    ( (__param(4, IInstantiationService)))
], TypeHierarchyController)));
registerEditorContribution(TypeHierarchyController.Id, TypeHierarchyController, 0 );
registerAction2(class PeekTypeHierarchyAction extends EditorAction2 {
    constructor() {
        super({
            id: 'editor.showTypeHierarchy',
            title: ( localize2(4256, 'Peek Type Hierarchy')),
            menu: {
                id: MenuId.EditorContextPeek,
                group: 'navigation',
                order: 1000,
                when: ( (ContextKeyExpr.and(_ctxHasTypeHierarchyProvider, PeekContext.notInPeekEditor))),
            },
            precondition: ( (ContextKeyExpr.and(_ctxHasTypeHierarchyProvider, PeekContext.notInPeekEditor))),
            f1: true
        });
    }
    async runEditorCommand(_accessor, editor) {
        return TypeHierarchyController.get(editor)?.startTypeHierarchyFromEditor();
    }
});
registerAction2(class extends EditorAction2 {
    constructor() {
        super({
            id: 'editor.showSupertypes',
            title: ( localize2(4257, 'Show Supertypes')),
            icon: Codicon.typeHierarchySuper,
            precondition: ( (ContextKeyExpr.and(
                _ctxTypeHierarchyVisible,
                 (_ctxTypeHierarchyDirection.isEqualTo("subtypes" ))
            ))),
            keybinding: {
                weight: 200 ,
                primary: 1024  + 512  + 38 ,
            },
            menu: {
                id: TypeHierarchyTreePeekWidget.TitleMenu,
                when: ( (_ctxTypeHierarchyDirection.isEqualTo("subtypes" ))),
                order: 1,
            }
        });
    }
    runEditorCommand(_accessor, editor) {
        return TypeHierarchyController.get(editor)?.showSupertypes();
    }
});
registerAction2(class extends EditorAction2 {
    constructor() {
        super({
            id: 'editor.showSubtypes',
            title: ( localize2(4258, 'Show Subtypes')),
            icon: Codicon.typeHierarchySub,
            precondition: ( (ContextKeyExpr.and(
                _ctxTypeHierarchyVisible,
                 (_ctxTypeHierarchyDirection.isEqualTo("supertypes" ))
            ))),
            keybinding: {
                weight: 200 ,
                primary: 1024  + 512  + 38 ,
            },
            menu: {
                id: TypeHierarchyTreePeekWidget.TitleMenu,
                when: ( (_ctxTypeHierarchyDirection.isEqualTo("supertypes" ))),
                order: 1,
            }
        });
    }
    runEditorCommand(_accessor, editor) {
        return TypeHierarchyController.get(editor)?.showSubtypes();
    }
});
registerAction2(class extends EditorAction2 {
    constructor() {
        super({
            id: 'editor.refocusTypeHierarchy',
            title: ( localize2(4259, 'Refocus Type Hierarchy')),
            precondition: _ctxTypeHierarchyVisible,
            keybinding: {
                weight: 200 ,
                primary: 1024  + 3
            }
        });
    }
    async runEditorCommand(_accessor, editor) {
        return TypeHierarchyController.get(editor)?.startTypeHierarchyFromTypeHierarchy();
    }
});
registerAction2(class extends EditorAction2 {
    constructor() {
        super({
            id: 'editor.closeTypeHierarchy',
            title: ( localize(4260, 'Close')),
            icon: Codicon.close,
            precondition: _ctxTypeHierarchyVisible,
            keybinding: {
                weight: 200  + 10,
                primary: 9 ,
                when: ContextKeyExpr.not('config.editor.stablePeek')
            },
            menu: {
                id: TypeHierarchyTreePeekWidget.TitleMenu,
                order: 1000
            }
        });
    }
    runEditorCommand(_accessor, editor) {
        return TypeHierarchyController.get(editor)?.endTypeHierarchy();
    }
});
